#include "include/common.h"

void init_config()
{
	extern struct config_t	config;
	extern struct func_t	func;

	memset( &config, 0, sizeof(struct config_t) );
	memset( &func, 0, sizeof(struct func_t) );

	config.filepath =		ENV_DEF_FILEPATH;

	config.server_port =		ENV_DEF_PORT;
	config.max_session =		ENV_DEF_MAX_SESSION;		//

	config.poll_timer =		ENV_DEF_POLL_TIMEOUT * 1000;
	config.access_control =		ENV_DEF_ACCESS_CONTROL;

	config.ssl =			ENV_DEF_SSL;
	config.ssl_cert =		ENV_DEF_SSL_CERT;
	config.ssl_key =		ENV_DEF_SSL_KEY;

	config.server_user =		ENV_DEF_USER;

	config.buf_size_header =	ENV_DEF_BUF_HEADER;
	config.buf_size_recv =		ENV_DEF_BUF_RECV;
	config.buf_size_send =		ENV_DEF_BUF_SEND;

	config.db_path =		ENV_DEF_DB_PATH;
	config.rep_path =		ENV_DEF_DB_REP_PATH;		//

	config.http_use =		ENV_DEF_HTTP_USE;
	config.http_root =		ENV_DEF_HTTP_ROOT;
	config.http_timeout =		ENV_DEF_HTTP_TIMEOUT;

	config.cache_use =		ENV_DEF_CACHE;
	config.cache_slot =		ENV_DEF_CACHE_SLOT;
	config.cache_size =		ENV_DEF_CACHE_SIZE;

	config.webdav =			ENV_DEF_WEBDAV;

	config.log_path =		ENV_DEF_LOG_PATH;		//
	
	return;
}

int parse_config( int argc, char **argv )
{
	extern struct config_t		config;
	
	int				c;		//getopt
	extern char			*optarg;	//getopt
	extern int			optind;		//getopt
	
	regex_t				reg_node;
	regmatch_t			node[5];
	size_t				node_cnt = 5;
	int				match_res;
	int				found = ENV_MAX;
	
	FILE				*conf;
	char				line[1024];

	struct config_node_t		set[ENV_MAX];
	int				set_len = 0;

	optind = 1;
	while ( (c = getopt( argc, argv, "c:hvsp:d:" )) > 0 )
	{
		switch ( c )
		{
			case 'c':
				config.filepath = malloc( 1024 );
				config.filepath = strncpy( config.filepath, optarg, 1024 );
			break;
			case 'h':
#ifdef	ENABLE_SSL
				printf( "Usage: %s [-c configfile] [-p port] [-d datadir] [-s]\r\n -c {configfile}	Location of config file\r\n -p {port}		Port number\r\n -d {datadir}		Location of data directory\r\n -s			Support SSL\r\n", basename(argv[0]) );
#else
				printf( "Usage: %s [-c configfile] [-p port] [-d datadir]\r\n -c {configfile}	Location of config file\r\n -p {port}		Port number\r\n -d {datadir}		Location of data directory\r\n", basename(argv[0]) );
#endif				
				return -1;
			break;
			case 'v':
				printf( "%s\n", SERVER_SOFTWARE );
				return -1;
			break;
		}
	}

	conf = fopen( config.filepath, "r" );
	if ( conf == NULL )
	{
		chdir( __PREFIX );
		conf = fopen( config.filepath, "r" );
		if ( conf == NULL )
		{
			perror("Cannot open config file");
			return -1;
		}
	}

	regcomp( &reg_node,	"([^=]*)\\s*=\\s*((\"[^\"]*\")|(.*))", REG_EXTENDED | REG_ICASE );
	while ( feof( conf ) == 0 && found > 0 )
	{
		fgets( line, 1024, conf );
		match_res = regexec( &reg_node, line, node_cnt, node, 0 );

		if ( match_res == 0 )
		{
			if ( line[node[0].rm_so] == '#' )
				continue;

			//printf("(%lld, %lld, %lld, %lld, %lld, %lld, %lld, %lld, %lld, %lld)%s", node[0].rm_so, node[0].rm_eo, node[1].rm_so, node[1].rm_eo, node[2].rm_so, node[2].rm_eo, node[3].rm_so, node[3].rm_eo, node[4].rm_so, node[4].rm_eo, line);

			set[set_len].buf = malloc( node[0].rm_eo+1 );
			snprintf( set[set_len].buf, node[0].rm_eo, "%s", line );

			set[set_len].field = &(set[set_len].buf[node[1].rm_so]);
			set[set_len].buf[node[1].rm_eo] = '\0';
			set[set_len].field_len = node[1].rm_eo-node[1].rm_so;

			set[set_len].value = &(set[set_len].buf[node[2].rm_so]);
			set[set_len].buf[node[2].rm_eo] = '\0';
			set[set_len].value_len = node[2].rm_eo-node[2].rm_so;

			if ( set[set_len].value[0] == '"' && set[set_len].value[set[set_len].value_len-2] == '"' )
			{
				set[set_len].value_len--;
				set[set_len].value[set[set_len].value_len-1] = '\0';
				set[set_len].value++;
			}

			set_len++;
			found--;
		}
	}
	regfree( &reg_node );

	fclose( conf );

	config.server_port = get_config_int( set, set_len, "port", config.server_port );
	config.max_session = get_config_int( set, set_len, "max session", config.max_session );
	config.poll_timer = get_config_int( set, set_len, "poll timer", config.poll_timer );
	config.buf_size_header = get_config_int( set, set_len, "header buf size", config.buf_size_header );
	config.buf_size_recv = get_config_int( set, set_len, "recv buf size", config.buf_size_recv );
	config.buf_size_send = get_config_int( set, set_len, "send buf size", config.buf_size_send );
	config.access_control = get_config_bool( set, set_len, "access control", config.access_control );

	config.http_timeout = get_config_int( set, set_len, "timeout", config.http_timeout );
	if ( config.http_timeout <= 0 )
	{
		config.http_timeout = 0;
		config.http_keeptime = 60;
	}
	else
		config.http_keeptime = config.http_timeout;

	config.ssl = get_config_bool( set, set_len, "ssl", config.ssl );
	config.ssl_cert = get_config_char( set, set_len, "ssl cert", config.ssl_cert );
	config.ssl_key = get_config_char( set, set_len, "ssl key", config.ssl_key );

	config.server_user = get_config_char( set, set_len, "server user", config.server_user );
	config.db_path = get_config_char( set, set_len, "data dir", config.db_path );
	config.rep_path = get_config_char( set, set_len, "repl dir", config.rep_path );
	config.http_root = get_config_char( set, set_len, "http document root", config.http_root );
	config.log_path = get_config_char( set, set_len, "log dir", config.log_path );

	config.cache_use = get_config_bool( set, set_len, "cache use", config.cache_use );
	config.cache_slot = get_config_int( set, set_len, "cache slot", config.cache_slot );
	config.cache_size = get_config_int( set, set_len, "cache size", config.cache_size );

	config.http_use = get_config_bool( set, set_len, "http use", config.http_use );
	config.webdav = get_config_bool( set, set_len, "webdav use", config.webdav );

	/* shared object */
	config.so_path = get_config_char( set, set_len, "shared object", NULL );
	if ( config.so_path != NULL )
	{
		config.func_after_fork = get_config_char( set, set_len, "function after fork", NULL );
		config.func_after_accept = get_config_char( set, set_len, "function after accept", NULL );
		config.func_after_thread = get_config_char( set, set_len, "function after thread", NULL );
		config.func_after_read = get_config_char( set, set_len, "function after read", NULL );
		config.func_on_post = get_config_char( set, set_len, "function on post", NULL );
		config.func_on_get = get_config_char( set, set_len, "function on get", NULL );
		config.func_on_webdav = get_config_char( set, set_len, "function on webdav", NULL );
		config.func_after_proc = get_config_char( set, set_len, "function after_proc", NULL );
		config.func_on_thread_exit = get_config_char( set, set_len, "function on thread exit", NULL );
		config.func_on_finalize = get_config_char( set, set_len, "function on finalize", NULL );
		
		if ( init_shared_object() < 0 )
			return -1;
	}

	optind = 1;
	while ( (c = getopt( argc, argv, "c:hvsp:d:" )) > 0 )
	{
		switch ( c )
		{
			case 'p':
				config.server_port = atoi(optarg);
			break;
			case 'd':
				config.db_path = malloc( 1024 );
				config.db_path = strncpy( config.db_path, optarg, 1024 );
			break;
			case 's':
				config.ssl = TRUE;
			break;
		}
	}

	for ( ; set_len--; )
		free( set[set_len].buf );

	return 0;
}

int init_shared_object()
{
	extern struct config_t	config;
	extern struct func_t	func;
	
	char			*err;

	func.dl_handle = dlopen( config.so_path, RTLD_NOW );
	if ( func.dl_handle == NULL )
		return init_fail_so();

	if ( config.func_after_fork != NULL )
	{
		func.after_fork = dlsym( func.dl_handle, config.func_after_fork );
		if ( (err = dlerror()) != NULL )
			return init_fail_so();
	}

	if ( config.func_after_accept != NULL )
	{
		func.after_accept = dlsym( func.dl_handle, config.func_after_accept );
		if ( (err = dlerror()) != NULL )
			return init_fail_so();
	}

	if ( config.func_after_thread != NULL )
	{
		func.after_thread = dlsym( func.dl_handle, config.func_after_thread );
		if ( (err = dlerror()) != NULL )
			return init_fail_so();
	}

	if ( config.func_after_read != NULL )
	{
		func.after_read = dlsym( func.dl_handle, config.func_after_read );
		if ( (err = dlerror()) != NULL )
			return init_fail_so();
	}

	if ( config.func_on_post != NULL )
	{
		func.on_post = dlsym( func.dl_handle, config.func_on_post );
		if ( (err = dlerror()) != NULL )
			return init_fail_so();
	}

	if ( config.func_on_get != NULL )
	{
		func.on_get = dlsym( func.dl_handle, config.func_on_get );
		if ( (err = dlerror()) != NULL )
			return init_fail_so();
	}

	if ( config.func_on_webdav != NULL )
	{
		func.on_webdav = dlsym( func.dl_handle, config.func_on_webdav );
		if ( (err = dlerror()) != NULL )
			return init_fail_so();
	}

	if ( config.func_after_proc != NULL )
	{
		func.after_proc = dlsym( func.dl_handle, config.func_after_proc );
		if ( (err = dlerror()) != NULL )
			return init_fail_so();
	}

	if ( config.func_on_thread_exit != NULL )
	{
		func.on_thread_exit = dlsym( func.dl_handle, config.func_on_thread_exit );
		if ( (err = dlerror()) != NULL )
			return init_fail_so();
	}

	if ( config.func_on_finalize != NULL )
	{
		func.on_finalize = dlsym( func.dl_handle, config.func_on_finalize );
		if ( (err = dlerror()) != NULL )
			return init_fail_so();
	}
	
	return 0;
}

int init_fail_so()
{
	extern struct func_t	func;
	
	perror( dlerror() );
	if ( func.dl_handle != NULL )
		dlclose( func.dl_handle );
	return -1;
}

char *get_config_char( struct config_node_t *set, int set_len, char *find, char *def )
{
	int		i;
	int		len;

	len = strlen( find );

	for ( i=set_len; i--; )
	{
		if ( strncasecmp( set[i].field, find, len ) == 0 )
			return set[i].value;
	}

	return def;
}

int get_config_int( struct config_node_t *set, int set_len, char *find, int def )
{
	int		i;
	int		len;

	len = strlen( find );

	for ( i=set_len; i--; )
	{
		if ( strncasecmp( set[i].field, find, len ) == 0 )
			return atoi(set[i].value);
	}

	return def;
}

short get_config_bool( struct config_node_t *set, int set_len, char *find, short def )
{
	int		i;
	int		len;

	len = strlen( find );

	for ( i=set_len; i--; )
	{
		if ( strncasecmp( set[i].field, find, len ) == 0 )
		{
			if ( strncmp( set[i].value, "1", 1 ) == 0 || strncasecmp( set[i].value, "true", 4 ) == 0 )
				return TRUE;
			else
				return FALSE;
		}
	}

	return def;
}

