//main.c
int client_sock_nonblock( int sock );
short check_catalog_exists();
void safe_exit( int sig );

//proc.c
void *client( void *data );
int client_auth_proc( struct thread_t *self );
int client_open_db( struct thread_t *self );
int client_process( struct thread_t *self, char *query );
void client_session_init( struct thread_t *self );
void client_safe_exit( struct thread_t *self );

//request.c
int request_header_parse( struct request_t *request );
void request_rtrim( char *buf );
int request_end( struct request_t *request );
short request_keep_alive( struct request_t *request );
int base64_decode( char *dst, const char *src, unsigned int len );
int request_decode( char *to, char *from, int from_len );
int request_decode_hexit( char c );

//admin.c
int admin_process( struct thread_t *self, int db_open_res );
int admin_create_user( struct db_user_t *user_data );
int admin_drop_user( struct db_user_t *user_data );
int admin_alter_user( struct db_user_t *user_data, int method );
int admin_user_manage( struct db_user_t *user_data, int method );

//extendsql.c
int extend_process( struct thread_t *self, int db_open_res );
int extend_desc( struct thread_t *self, char *tablename );
int extend_show_database( struct thread_t *self );
int extend_show_user( struct thread_t *self );

//libnetwork.c
int server_sock_init( sock_t *server_addr, short port, int backlog );
int net_send( int sock, char *buf, int len );
int net_sendfile( int sock, int filefd, off_t offset, ssize_t size );

#ifdef	ENABLE_SSL
int send_via_ssl( int sock, SSL *ssl, char *buf, int len );
int net_send_ssl( int sock, SSL *ssl, char *buf, int len );
int net_sendfile_ssl( int sock, SSL *ssl, int filefd, off_t offset, ssize_t size );
#endif

//libsqlite3.c
int db_open( struct db_t *db, char *filename );
int db_open_init( struct db_t *db, char *filename, char *init_sql );
int db_close( struct db_t *db );
int db_set_authorizer( struct db_t *db, void *data );
int db_throw( struct db_t *db, char *sql );
int db_commit( struct db_t *db );
int db_rollback( struct db_t *db );
int db_changes( struct db_t *db );
long long int db_last_rowid( struct db_t *db );
int db_table_columns( struct db_t *db );
int db_table_column_name( char **column_name, struct db_t *db, int offset );
int db_get_table( struct db_t *db, char *sql );
int db_table_num_rows( struct db_t *db );
int db_table_has_rows( struct db_t *db );
char *db_table_result( struct db_t *db, int row, int col );
void db_free_table( struct db_t *db );
void _db_free_table( char **table );
char *db_printf( const char *sql, ...);
void db_err_free( struct db_t *db );
void db_free( char *sql );
const char *db_version();
int db_statement( struct db_t *db, char *sql );
int db_statement_column_count( struct db_t *db );
const char *db_statement_column_type( struct db_t *db, int seq );
const char *db_statement_column_name( struct db_t *db, int seq );
int db_statement_free( struct db_t *db );
int db_column_meta( struct db_t *db, char *table, const char *column, struct desc_t *desc );
const char *db_error( struct db_t *db );

//response.c
int response( struct thread_t *self );
int response_buf_open( char *buf, size_t size, int changes, long long int rowid, int rows, struct timeval *start );
int response_flush( struct thread_t *self, short is_last );
int response_error( struct thread_t *self, int status, char *errmsg );
int response_info( struct thread_t *self, int status, char *infomsg );
int response_message( struct thread_t *self, char *msg );
int response_xml_printf( char *dst, const char *src, const int src_len );

//http.c
int http_proc( struct thread_t *self );
time_t http_asctime2time_t( char *asctime );
int http_ascmon2int( char *ascmonth );
int http_time2asctime( char *dst, time_t time );
int webdav_proc( struct thread_t *self );

//config.c
void init_config();
int parse_config( int argc, char **argv );
char *get_config_char( struct config_node_t *set, int set_len, char *find, char *def );
int get_config_int( struct config_node_t *set, int set_len, char *find, int def );
short get_config_bool( struct config_node_t *set, int set_len, char *find, short def );

int init_shared_object();
int init_fail_so();

//monitor.c
void monitor_thread_push( struct thread_t *self );
void monitor_thread_delete( struct thread_t *self );
int monitor_response( struct thread_t *self );

//auth.c
int auth_init( char *admin_id, char *admin_pass );
int auth_user_flush();
int auth_db_flush();
int auth_object_flush();
int authorizer( void *data, int auth_code, const char *argv1, const char *argv2, const char *dbname, const char *object );
int auth_update( short type, struct grant_t *data );
int auth_create_update( short type, struct grant_t *data );
int auth_user_check( char *id, char *pass, struct db_user_t *result );
char *auth_db_check( const char *id, const char *db, int grant_type );
char *auth_object_check( const char *id, const char *db, const char *object, int grant_type );

//pstmt.c
int pstmt_process( struct thread_t *self );
int pstmt_init( struct thread_t *self, unsigned long pstmt_id, char *sql );
int pstmt_release( struct thread_t *self, unsigned long pstmt_id );
int pstmt_reset( struct thread_t *self, unsigned long pstmt_id );
int pstmt_bind( struct thread_t *self, unsigned long pstmt_id, int bind_seq, char *bind_type, char *data, int data_len );
int pstmt_execute( struct thread_t *self, unsigned long pstmt_id );

int pstmt_internal_row( struct thread_t *self, struct pstmt_t *pstmt );
void pstmt_internal_gc( struct thread_t *self );
struct pstmt_t *pstmt_internal_find( struct thread_t *self, unsigned long pstmt_id );
int pstmt_internal_release( struct thread_t *self, unsigned long pstmt_id );

//cache.c
int cache_init( int cache_cnt, int buf_size );
void cache_set_update_hook( struct db_t *db );
void cache_release_hook( void *data, int action, char const *db_name, char const *table_name, sqlite_int64 rowid );
void cache_release( char *db_name );
void cache_push( char *db_name, char *query, long long int rowid, int rows, char *buf, int size );
int cache_get( char *db_name, struct cache_t *qcache );
struct cache_t *cache_get_old();
void cache_safe_exit();

